// AP(r) Computer Science Marine Biology Simulation:  This 
// class was added by Georgia Tech to extend past Fish
// The DrawableToolbar class was created based on the FishToolbar
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;
import java.awt.geom.AffineTransform;
import javax.swing.*;
import java.util.Iterator;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>DrawableToolbar</code> is a toolbar with controls for configuring 
 *  the specifications for a new drawable object to be created. It has
 *  two combo boxes, one listing the various drawable classes and another
 *  giving the color choices.
 *
 *  <p>
 *  The <code>DrawableToolbar</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @author Barb Ericson ericson@cc.gatech.edu
 *  @version 1 August 2002
 **/

public class DrawableToolbar extends JToolBar 
{
    private JComboBox colorComboBox, drawableComboBox;
    
   /** Creates a new DrawableToolbar and all its tools.
    **/
   public DrawableToolbar() 
    {
        super(SwingConstants.VERTICAL);
        setFloatable(false);
        makeTools();
    }
        
   /** Returns the currently selected Drawable class.
    *  @return a Drawable class
    **/
    public Class currentDrawableClass()
    {
        return ((DrawableChoice)drawableComboBox.getSelectedItem()).getDrawableClass();
    }
    
   /** Returns the currently selected color. If random color is selected,
    *  returns a new randomly generated color.
    *  @return the color to use for a new drawable object
    **/
    public Color currentColor()
    {
        ColorChoice cc = (ColorChoice)colorComboBox.getSelectedItem();
        if (cc == ColorChoice.random) 
        {
            Random rng = RandNumGenerator.getInstance();
            return new Color(rng.nextInt(256), rng.nextInt(256), rng.nextInt(256));
        }
        else
            return cc.getColor();
    }

    // creates the UI components for the toolbar
    private void makeTools() 
    {
        setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Configure new object"), 
                BorderFactory.createEmptyBorder(4, 4, 4, 4)));
        
        add(new JLabel(" Type: "));
        drawableComboBox = new JComboBox();
        DrawableChoice.addChoicesFromFactory(drawableComboBox);
        drawableComboBox.setRenderer(new ChoiceWithIconRenderer(drawableComboBox));
        drawableComboBox.setAlignmentX(LEFT_ALIGNMENT);
        drawableComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { drawableChosen(); }});
        add(drawableComboBox);
      
        addSeparator();
        
        add(new JLabel(" Color: "));
        colorComboBox = new JComboBox(ColorChoice.standardChoices);
        colorComboBox.setSelectedItem(ColorChoice.random);
        colorComboBox.setRenderer(new ChoiceWithIconRenderer(colorComboBox));
        colorComboBox.setAlignmentX(LEFT_ALIGNMENT);
        colorComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { colorChosen(); }});
        add(colorComboBox);

        add(Box.createGlue());
    }
    
    // callback messaged when the user picks a new choice from the color 
    // combo box.
    private void colorChosen()
    {
       ColorChoice cc = (ColorChoice)colorComboBox.getSelectedItem();
       if (cc == ColorChoice.custom) 
       {
            colorComboBox.hidePopup();
            Component parentFrame = JOptionPane.getFrameForComponent(this);
            Color chosen = JColorChooser.showDialog(parentFrame, "Choose Object Color", cc.getColor());
            if (chosen != null) cc.setColor(chosen);
        }
    }
    
    // callback messaged when the user picks a new choice from the drawable 
    // combo box.
    private void drawableChosen()
    {
        DrawableChoice fi = (DrawableChoice)drawableComboBox.getSelectedItem();
        colorComboBox.setEnabled(MBSFactory.hasFourArgCtor(fi.getDrawableClass()));
    }
    
    /** Interface that unifies choices with icons (drawable object, color) so
     *  we can use one common renderer for both.
     */
    private interface ChoiceWithIcon {
        Icon getIcon();
    }

    /** Nested class used to hold the per-item information 
     *  for the entries in the combo box of color
     *  choices. Each item represents a color choice which
     *  is basically just a Color object and a name.
     */
    private static class ColorChoice implements ChoiceWithIcon
    {
        public static final ColorChoice random = new ColorChoice("Random", ColorIcon.RANDOM_COLOR);
        public static final ColorChoice custom = new ColorChoice("Other ...", Color.white);
        
        private Color color;
        private String name;
        private ColorIcon icon;

        public ColorChoice(String n, Color c) { color = c; name = n; icon = new ColorIcon(color, 16, 16); }
        public void setColor(Color c) { color = c; icon.setColor(c); }
        public Color getColor() { return color; }
        public String toString() { return name; }
        public Icon getIcon() { return icon; }
        
        public static final ColorChoice[] standardChoices = { 
                    new ColorChoice("Red", Color.red),
                    new ColorChoice("Orange", new Color(255, 128, 0)),
                    new ColorChoice("Yellow", Color.yellow),
                    new ColorChoice("Green", Color.green),
                    new ColorChoice("Blue", new Color(0, 128, 255)),
                    new ColorChoice("Purple", new Color(128, 0, 128)),
                    ColorChoice.random, ColorChoice.custom };
    }
    
    
    /** Nested class used to hold the per-item information 
     *  for the entries in the combo box of drawable choices.
     *  Each item represents a drawable choice which is a drawable class.
     */
    private static class DrawableChoice implements ChoiceWithIcon
    {
        private Class cls;
        private DrawableIcon icon;
      
        public DrawableChoice(Class c) { cls = c; icon = 
          new DrawableIcon(cls, Color.gray, 16, 16); }
        public String toString() { return cls.getName(); }
        public Icon getIcon() { return icon; }
        public Class getDrawableClass() { return cls; }
        public static void addChoicesFromFactory(JComboBox cb)
        {
            Iterator iter = MBSFactory.envObjectClasses().iterator();
            while (iter.hasNext())
            {
                DrawableChoice fc = new DrawableChoice((Class)iter.next());
                cb.addItem(fc);
                if (fc.getDrawableClass() == Drawable.class) cb.setSelectedItem(fc);
            }
        }
    }
    
    // Custom cell renderer that displays text and icon.
    // (Default renderer can only display one or the other.)
    private static class ChoiceWithIconRenderer extends JLabel implements ListCellRenderer 
    {
        private JComboBox cb;
        
        public ChoiceWithIconRenderer(JComboBox b) 
        {
            setOpaque(true);
            setHorizontalAlignment(LEFT);
            setVerticalAlignment(CENTER);
            setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
            cb = b;
        }
        
        public Component getListCellRendererComponent(JList list,
                    Object value, int index,
                    boolean selected, boolean cellHasFocus) 
        {
            setBackground(selected ? list.getSelectionBackground() : list.getBackground());
            setForeground(selected ? list.getSelectionForeground() : list.getForeground());
            if (!cb.isEnabled())
            {
                setText("No choice"); // draw differently when disabled
                setIcon(null);
            } 
            else 
            {
                setText(value.toString());
                setIcon(((ChoiceWithIcon)value).getIcon());
            }
            return this;
        }
    }
    
    
    /** Nested class used to draw the color swatch icon used
     *  for color choice entries in the color combo box.
     *  This simple class just draws a rectangle filled with the
     *  color and edged with a black border.
     */
    private static class ColorIcon implements Icon 
    {
        // so we have a unique object to compare == to
        public static final Color RANDOM_COLOR = new Color(0, 0, 0);
        private static final int MARGIN = 2;
        
        private Color color;
        private int width, height;
    
        public ColorIcon(Color c, int w, int h) { color = c; width = w; height = h; }
        public void setColor(Color c) { color = c; }
        public int getIconWidth() { return width; }
        public int getIconHeight() { return height; }
        public void paintIcon(Component comp, Graphics g, int x, int y) 
        {
            Graphics2D g2 = (Graphics2D)g;
            Rectangle r = new Rectangle(x + MARGIN, y + MARGIN, width - 2*MARGIN, height - 2*MARGIN);
            if (color != RANDOM_COLOR) 
            {
                g2.setColor(color);
                g2.fill(r);
            } 
            else for (int k = 0; k < r.width; k++)  // draw rainbow lines
            {
                g2.setColor(Color.getHSBColor((float)k/r.width, .95f, 1));
                g2.drawLine(r.x + k, r.y, r.x + k, r.y + r.height);
            }
            g2.setColor(Color.black);
            g2.draw(r);
        }
    }
        
    /** Nested class used to draw the drawable icon used
     *  for drawable entries in the drawable combo box.
     *  We construct a drawable object and then hand it off to its
     *  display object to draw.
     */
    static class DrawableIcon implements Icon
    {
        private Drawable drawable;
        private Class cls;
        private Color color;
        private int width, height;

        public DrawableIcon(Class cl, Color c, int w, int h) { cls = cl; color = c; width = w; height = h; }
        public int getIconWidth() { return width; }
        public int getIconHeight() { return height; }
        public void paintIcon(Component comp, Graphics g, int x, int y) 
        {
            if (drawable == null)
            { 
                if (MBSFactory.hasFourArgCtor(cls))
                    drawable = (Drawable)MBSFactory.constructEnvObject(cls, new BoundedEnv(1, 1), new Location(0, 0), Direction.EAST, color);
                else
                    drawable = (Drawable)MBSFactory.constructEnvObject(cls, new BoundedEnv(1, 1), new Location(0, 0), Direction.EAST);
            }
            Graphics2D g2 = (Graphics2D)g;
            AffineTransform savedTransform = g2.getTransform(); // save current
            LocatableDisplay displayObj = DisplayMap.findDisplayFor(drawable);
            displayObj.draw(drawable, comp, g2, new Rectangle(x, y, getIconWidth(), getIconHeight()));
            g2.setTransform(savedTransform); // restore coordinate system
        }
    }
    
}

    
